/*
 * Enigma: an interpreter for the Enigma language
 * Copyright (C) 2010  Niels Serup

 * This file is part of Enigma.

 * Enigma is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Enigma is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Enigma.  If not, see <http://www.gnu.org/licenses/>.
 */

package enigma;

/**
 * The starter class
 * @author  Niels Serup <ns@metanohi.org>
 * @version 0.1
 */
public class EnigmaMain {
    private static EnigmaLogic logic;

    public static void main(String[] origArgs) throws Exception {
        boolean verbose = true;
        String filename = null;
        String[] args = null;
        
        boolean displayHelp = false;
        if (origArgs.length == 0)
            displayHelp = true;
        
        int i = 0;
        for (String x : origArgs) {
            i++;
            if (x.equals("-h") || x.equals("--help")) {
                displayHelp = true;
                break;
            }
            else if (x.equals("-v") || x.equals("--version")) {
                System.out.println(
"Enigma 0.1\n" + 
"Copyright (C) 2010  Niels Serup <ns@metanohi.org>\n" +
"License GPLv3+: GNU GPL version 3 or later\n" +
"<http://gnu.org/licenses/gpl.html>\n" +
"This is free software: you are free to change and redistribute it.\n" +
"There is NO WARRANTY, to the extent permitted by law.");
                System.exit(0);
            }
            else if (x.equals("-q") || x.equals("--quiet")) {
                verbose = false;
            }
            else {
                if (filename == null) {
                    if (x.codePointAt(0) == '\\')
                        x = x.substring(1);
                    filename = x;
                    args = new String[origArgs.length - i];
                    i = -1;
                }
                else
                    args[i] = x;
            }
        }
        
        if (displayHelp) {
            System.out.println(
"Usage: enigma [OPTION]... FILE [ARGUMENT]...\n" +
"Runs Enigma programs\n\n" +

"Options:\n" +
"  -q, --quiet     Do not print error messages\n" +
"  -h, --help      Show this help and exit\n" +
"  -v, --version   Show version information and exit");
            System.exit(0);
        }
                
        logic = new EnigmaLogic(verbose);       
        logic.parseFile(filename, args);
    }
}
