#!/usr/bin/env python

#   a curses frontend for simple chatting
#   Copyright (C) 2010  Niels Serup
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 3 of 
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <http://www.gnu.org/licenses/>.
########################################################################
## Maintainer: Niels Serup <ns@metanohi.org>

import curses

class SimpleChatFrontend:
    def __init__(self, parent):
        self.self = parent
        self.msgs = ['Type /exit to exit']
        self.exit_str = '/exit'
        self.temp_msg = ''
        self.temp_p = 0

    def start(self):
        self.screen = curses.initscr()
        self.max = self.screen.getmaxyx()
        curses.noecho()
        self.update()
        self.input_loop()

    def smart_getstr(self):
        screen = self.screen
        msgs = self.msgs
        x = -1
        self.temp_msg = ''
        local_msgs = {}
        loc = 0

        msgslist_num = 0
        BACKSPACE = 127

        self.temp_p = 0
        while 1:
            pla = loc - self.temp_p
            x = screen.getch(self.max[0]-1, pla)
            if x == 10: break

            updo = 0
            rile = 0
            np = 0
            delete = False

            if x == 27:
                y = screen.getch(self.max[0]-1, pla)
                if y == 91:
                    y = screen.getch(self.max[0]-1, pla)
                    if y == 65:
                        updo = 1
                    elif y == 66:
                        updo = -1
                    elif y == 67:
                        rile = 1
                    elif y == 68:
                        rile = -1
                    elif y == 51:
                        screen.getch(self.max[0]-1, pla)
                        delete = True
                elif y == self.max[1]-1:
                    y = screen.getch(self.max[0]-1, pla)
                    if y == 72:
                        np = -1
                    elif y == 70:
                        np = 1

            if updo is not 0:
                num = msgslist_num + updo
                if str(num) in local_msgs:
                    nmsg = local_msgs[str(num)]
                elif num > 0 and num < len(msgs):
                    nmsg = msgs[-num]
                else:
                    nmsg = None
                if nmsg is not None:
                    local_msgs[str(msgslist_num)] = self.temp_msg
                    self.temp_msg = nmsg
                    msgslist_num = num
                    self.clear_input()
                    loc = len(self.temp_msg)
            elif rile is not 0:    
                loc += rile
                if loc < 0: loc = 0
                elif loc > len(self.temp_msg): loc = len(self.temp_msg)
            elif np is not 0:
                if np == -1:
                    loc = 0
                elif np == 1:
                    loc = len(self.temp_msg)
            elif x == BACKSPACE:
                if loc > 0:
                    self.temp_msg = self.temp_msg[:loc-1] + self.temp_msg[loc:]
                    loc -= 1
                    self.clear_input()
            elif delete:
                if loc < len(self.temp_msg):
                    self.temp_msg = self.temp_msg[:loc] + self.temp_msg[loc+1:]
                    self.clear_input()
            else:
                try: k = chr(x)
                except Exception: continue
                self.temp_msg = self.temp_msg[:loc] + k + self.temp_msg[loc:]
                loc += 1
                #m=str(x); self.temp_msg +=','+ m; loc += len(m)+1
            self.temp_p = loc - self.max[1] + 1
            if self.temp_p < 0:
                self.temp_p = 0
            self.print_input()

        msg = self.temp_msg
        self.clear_input()
        self.temp_msg = ""
        return msg

    def write(self, msg):
        self.msgs.append(msg)
        self.update()

    def clear_input(self):
        self.screen.addstr(self.max[0]-1, 0, " " * (self.max[1]-1))

    def print_input(self):
        tmsg = self.temp_msg[self.temp_p:self.temp_p+self.max[1]-1]
        self.screen.addstr(self.max[0]-1, 0, tmsg)

    def update(self):
        screen = self.screen
        self.max = screen.getmaxyx()
        screen.clear()

        i = len(self.msgs) - 1
        cur = self.max[0] - 2
        while cur > 0 and i > -1:
            msg = self.msgs[i]
            linelen = (len(msg) - 1) / self.max[1] + 1
            cur -= linelen
            screen.addstr(cur, 0, msg)
            i -= 1
        self.print_input()
        screen.refresh()

    def input_loop(self):
        while 1:
            #msg = screen.getstr(self.max[0]-1, 0)
            msg = self.smart_getstr()
            if msg == self.exit_str: break
            if not msg: continue
            self.self.send(msg)

        self.end()

    def end(self):
        curses.endwin()

if __name__ == '__main__':
    class FakeClient:
        def start(self):
            self.frnt = SimpleChatFrontend(self)
            self.frnt.start()

        def send(self, msg):
            self.frnt.write(msg)

        def end(self):
            self.frnt.end()

    fake = FakeClient()
    try:
        fake.start()
    except (KeyboardInterrupt, EOFError):
        pass
    finally:
        fake.end()

