#!/usr/bin/env python

#   simplechatclient: a very simple chat client
#   Copyright (C) 2010  Niels Serup
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 3 of 
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <http://www.gnu.org/licenses/>.
########################################################################
## Maintainer: Niels Serup <ns@metanohi.org>

import socket
import sys
from quickthread import dthread as thread
from frontend import SimpleChatFrontend


class SimpleChatClient:
    def __init__(self, host='localhost', port=51008,
                 frontend_class=SimpleChatFrontend):
        if host is None: host = 'localhost'
        if port is None: port = 51008
        self.host = host
        self.port = port
        self.frontend_class = frontend_class

    def start(self):
        # Create a socket (SOCK_STREAM means a TCP socket)
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect((self.host, self.port))

        self.frnt = self.frontend_class(self)

        thread(self.recv_loop)
        self.frnt.start()

    def send(self, msg):
        try:
            self.s.send(msg)
        except Exception:
            self.s.close()

    def recv_loop(self):
        while 1:
            msg = self.s.recv(1024)
            self.frnt.write(msg)
        self.end()

    def end(self):
        self.s.close()
        self.frnt.end()

if __name__ == '__main__':
    # Run it
    from optparse import OptionParser
    parser = OptionParser(prog='simplechatclient',
                          usage='usage: %prog HOST PORT',
                          description='Runs a simple chat client', version="""\
%prog 0.93
Copyright (C) 2010  Niels Serup
License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.""")
    (options, args) = parser.parse_args()
    args = args[:2]
    if len(args) < 2:
        args.append(None)
        if len(args) < 2:
            args.append(None)
    try: args[1] = int(args[1])
    except Exception: pass
    args.append(SimpleChatFrontend)
    cl = SimpleChatClient(*args)
    try:
        cl.start()
    except (KeyboardInterrupt, EOFError):
        pass
    finally:
        cl.end()
