#!/usr/bin/env python

#   simplechatserver: a very simple chat server
#   Copyright (C) 2010  Niels Serup
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU Affero General Public 
#   License as published by the Free Software Foundation, either
#   version 3 of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero
#   General Public License along with this program.
#   If not, see <http://www.gnu.org/licenses/>.
########################################################################
## Maintainer: Niels Serup <ns@metanohi.org>

import socket
from quickthread import dthread as thread


class SimpleChatServer:
    def __init__(self, port=51008):
        if port is None: port = 51008
        self.port = port
        self.conns = []
        self.dats = []
        self.receiving_now = {}
        self.done = False

    def start(self):
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.bind(('', self.port))
        self.s.listen(1)

        thread(self.accept_conns)
        thread(self.receive_all)

        self.logic()

    def accept_conns(self):
        while not self.done:
            conn, addr = self.s.accept()
            self.receiving_now[conn] = False
            self.conns.append(conn)

    def recv(self, x):
        data = x.recv(1024)
        if not data:
            try:
                self.conns.remove(x)
            except Exception:
                pass
            return False
        print 'Received', repr(data)
        if data:
            self.dats.append(data)
        self.receiving_now[x] = False

    def receive_all(self):
        while not self.done:
            for x in self.conns:
                if not self.receiving_now[x]:
                    self.receiving_now[x] = True
                    thread(self.recv, x)

    def logic(self):
        while 1:
            if not self.dats: continue
            t = self.dats[0]
            print 'Sending', t
            for x in self.conns:
                try:
                    x.send(t)
                except Exception:
                    try:
                        self.conns.remove(x)
                    except Exception:
                        pass
            self.dats.pop(0)
        self.end()

    def end(self):
        self.done = False # Also ends threads
        for x in self.conns:
            x.close()
        self.s.close()

if __name__ == '__main__':
    # Run it
    from optparse import OptionParser
    parser = OptionParser(prog='simplechatserver', usage='usage: %prog PORT',
                          description='Runs a simple chat server', version="""\
%prog 0.93
Copyright (C) 2010  Niels Serup
License AGPLv3+: GNU AGPL version 3 or later <http://gnu.org/licenses/agpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.""")
    (options, args) = parser.parse_args()
    args = args[:1]
    try: args[0] = int(args[0])
    except Exception: pass
    cs = SimpleChatServer(*args)
    try:
        cs.start()
    except (KeyboardInterrupt, EOFError):
        print
    finally:
        cs.end()
