#!/usr/bin/env python
# -*- coding: utf-8 -*-

# vit: a tool for keeping a logbook
# Copyright (C) 2010  Niels Serup

# This file is part of vit.
#
# vit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# vit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with vit.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## vit.database
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Functions for managing vit databases

import sqlite3

class Database(object):
    def __init__(self, fname):
        self.fname = fname

    def start(self):
        self.db = sqlite3.connect(self.fname, detect_types=sqlite3.PARSE_DECLTYPES)
        self.cr = self.db.cursor()

    def execute(self, sql, *parameters):
        return self.cr.execute(sql, parameters)
    
    def create(self):
        try:
            self.execute('create table messages (\
id integer primary key autoincrement, \
message text, begin_time timestamp, end_time timestamp)')
        except sqlite3.OperationalError:
            raise Exception('database already exists')

    def add(self, msg, begin, end):
        try:
            return self.execute('insert into messages (id, message, \
begin_time, end_time) values (NULL,?,?,?)', msg, begin, end)
        except sqlite3.OperationalError:
            raise Exception('no database exists')

    def get(self):
        try:
            return self.execute('select message, begin_time, end_time \
from messages order by id desc')
        except sqlite3.OperationalError:
            raise Exception('no database exists')

    def end(self):
        self.db.commit()
