#!/usr/bin/env python
# -*- coding: utf-8 -*-

# vit: a tool for keeping a logbook
# Copyright (C) 2010  Niels Serup

# This file is part of vit.
#
# vit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# vit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with vit.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## vit.various
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Various minor (but still important) parts

import sys
import subprocess
import vit.generalinformation as ginfo
try:
    from termcolor import colored
except ImportError:
    from vit.external.termcolor import colored

def error(msg, done=None, pre=None, **kwds):
    errstr = str(msg) + '\n'
    if pre is not None:
        errstr = pre + ': ' + errstr
    if kwds.get('use_color'):
        errstr = colored(errstr, 'red')
    sys.stderr.write(errstr)
    if done is not None:
        if done in (True, False):
            sys.exit(1)
        else:
            sys.exit(done)

def _usable_error(msg, done=None, **kwds):
    error(msg, done, ginfo.program_name + ': ', **kwds)

def status(msg, pre=None, **kwds):
    statstr = str(msg)
    if pre is not None:
        statstr = pre + statstr
    if kwds.get('use_color'):
        statstr = colored(statstr, 'blue')
    print statstr

def _usable_status(msg, **kwds):
    status(msg, ginfo.program_name + ': ', **kwds)

def exec_program(*args):
    return subprocess.call(map(str, args))
