/*
    wordwork: a system meant to create sentences using random words
    Copyright (C) 2010  Niels Serup

    This file is part of wordwork.

    wordwork is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wordwork is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with wordwork.  If not, see <http://www.gnu.org/licenses/>.

    Maintained by Niels Serup <ns@metanohi.org>
*//*
      ===========================================
    || Documentation and downloads are available ||
    || at http://metanohi.org/projects/wordwork/ ||
      ===========================================
  */

// Includes
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Variables
char *filename;
FILE *f;
int current_char;
short tmp_title_len;
short in_title;

// Parse function
void parse() {
  f = fopen(filename, "r");
  tmp_title_len = 0;
  in_title = 0;
  
  while ((current_char = getc(f)) != EOF) {
    if (in_title == 0 && current_char == '<')
      in_title = 1;
    else if (in_title == 1) {
      if (current_char == 't')
        in_title = 2;
      else
        in_title = 0;
    }
    else if (in_title == 2) {
      if (current_char == 'i')
        in_title = 3;
      else
        in_title = 0;
    }
    else if (in_title == 3) {
      if (current_char == 't')
        in_title = 4;
      else
        in_title = 0;
    }
    else if (in_title == 4) {
      if (current_char == 'l')
        in_title = 5;
      else
        in_title = 0;
    }
    else if (in_title == 5) {
      if (current_char == 'e')
        in_title = 6;
      else
        in_title = 0;
    }
    else if (in_title == 6) {
      if (current_char == '>')
        in_title = 7;
      else
        in_title = 0;
    }
    else if (in_title == 7) {
      if (current_char == ':') {
        while (tmp_title_len > 0) {
          putchar('\b'); // Delete one printed character
          tmp_title_len--;
        }
        in_title = 0;
      }
      else if (current_char == '<') {
        putchar('\n'); // Newline
        tmp_title_len = 0;
        in_title = 0;
      }
      else {
        tmp_title_len += 1;
        putchar(current_char); // Print character
      }
    }
  }
  
  fclose(f);
}

int main(int argc, char *argv[]) {
  if (argc == 1) {
    fprintf(stderr, "Error: No file specified.\n");
    return EXIT_FAILURE;
  }

  if (argc == 2 && (strcmp(argv[1], "--help") == 0 || strcmp(argv[1], "-h") == 0)) {
    printf("Usage: %s WIKIFILE\n"
      "Outputs a newline-separated list of all words in the file.\n\n"
      "Options:\n"
      "   -h, --help   show this help and exit\n"
      "   --version    show version information and exit\n\n"
      "In other words, it parses XML files with the syntax used by"
      " e.g. Wiktionary.\n",
      argv[0]);
    return 0;
  }

  if (argc == 2 && strcmp(argv[1], "--version") == 0) {
    printf("wordwork 0.1\n"
"Copyright (C) 2010  Niels Serup <ns@metanohi.org>\n"
"License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n"
"This is free software: you are free to change and redistribute it.\n"
"There is NO WARRANTY, to the extent permitted by law.\n");
    return 0;
  }

  if (argc == 2)
    filename = argv[1];
  else
    filename = argv[2];

  parse();
  return 0;
}
