#!/usr/bin/env python

#   wordwork: a system meant to create sentences using random words
#   Copyright (C) 2010  Niels Serup
#
#   This file is part of wordwork.
#
#   wordwork is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   wordwork is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with wordwork.  If not, see <http://www.gnu.org/licenses/>.
#
#   Maintained by Niels Serup <ns@metanohi.org>
#
#     ===========================================
#   || Documentation and downloads are available ||
#   || at http://metanohi.org/projects/wordwork/ ||
#     ===========================================

### SERVER ###

# Imports
import sys
import socket
from datetime import datetime

class NullOut:
    def write(self, msg):
        pass

class ModOut:
    def __init__(self, quiet=False):
        if not quiet:
            self.out = sys.stdout
        else:
            self.out = NullOut()
        sys.stdout = self

    def write(self, msg):
        if msg == '\n':
            self.out.write(msg)
        else:
            self.out.write(str(datetime.today()) + '::::' + msg)

class WordworkServer:
    def __init__(self, data, port=None, quiet=False):
        if port is None: port = 51346
        ids = 'j', 'c', 'C', 's', 'S', 'd', 'D', 'v', 't', 'p', 'l', \
            'o', 'n', 'O', 'P', 'i'
        c = data.split('\n')

        data = {}
        for x in ids:
            data[x] = []

        for x in c[:-1]:
            data[x[0]].append(x[1:])
        del c

        size = {}
        for x in ids:
            size[x] = len(data[x])
        self.quiet = quiet
        self.sizestr = str(size)
        self.port = port
        self.data = data

    def start(self):
        ModOut(self.quiet)
        sizestr = self.sizestr
        data = self.data

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(('', self.port))

        print 'Serving...'

        while 1:
            s.listen(1)
            conn, addr = s.accept()
            print 'Connected by ' + str(addr)
            while 1:
                msg = conn.recv(1024)
                if not msg: break
                print 'Received ' + msg
                if msg == '!':
                    conn.send(sizestr)
                    print 'Sent sizes'
                elif len(msg) > 1:
                    msg = msg.split(';')
                    tmp = []
                    for x in msg:
                        try:
                            tmp.append(data[x[0]][int(x[1:])])
                        except:
                            tmp.append('?')
                    tmp = str(tmp)
                    conn.send(tmp)
                    print 'Sent ' + tmp
        conn.close()
        s.close()
