# Zita: A simple text adventure engine heavily relying on Python
# Copyright (C) 2009  Niels Serup
#  
#	This file is part of Zita.
#
#	Zita is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	Zita is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with Zita.  If not, see <http://www.gnu.org/licenses/>.

# defs.py will not work on its own, as it relies on several global
# variables defined in zita.py.

def p(t):
    print t

def loop_through_fs(arg, dir, elems):
    # This function loops through files and dirs in dir
    for x in elems:
        addr = (dir + '/' + x).replace('\\', '/')
        if os.path.isfile(addr):
            gamefiles.append(addr)
        elif os.path.isdir(addr):
            gamedirs.append(addr)

os.path.walk(gamedir, loop_through_fs, None) # Loop

mainfile = '' # Needed by defs.py -- value changes in a few lines

def liststrip(lst):
    # Uses the strip function on all strings in a list
    for x in range(len(lst)):
        lst[x] = lst[x].strip()
    return lst

def getfilenames(name, details = None):
    # Looks for files in game root dir and in game dirs
    
    global mainfile # Needed to make sure mainfile isn't reimported
    files = []
    
    if (not details or details.find('file') != -1):
        for x in gamefiles:
            if (x[len(gamedir) + 1:]).find(name) == 0 and (x[len(gamedir) + 1:]).find(name + '/') != 0 and x != mainfile and x[-1:] != '~':
                files.append(x)
        
        if details == 'file':
            # Attempts to find the best file
            if not files:
                return None
            elif len(files) == 1:
                file = files[0]
            else:
                if gamedir + '/' + name + '.' + filesuffix in files:
                    file = gamedir + '/' + name + '.' + filesuffix
                elif gamedir + '/' + name in files:
                    file = gamedir + '/' + name
                else:
                    file = files[0]
            
            return file
    
    # Find all files in dir (not recursive)
    if (not details or details.find('dir') != -1):
        for x in gamedirs:
            if (x[len(gamedir) + 1:] + '/').find(name + '/') == 0:
                for y in gamefiles:
                    if (y + '/').find(x + '/') == 0 and x != mainfile and x[-1:] != '~':
                        files.append(y)
    
    return files

def include(lst, d = None):
    # Imports files
    nlst = []
    for x in lst:
        nlst.extend(getfilenames(x, d))
    
    txt = 'global room, item, var, command, default'
    for x in nlst:
        txt += '\n' + open(x).read()
    exec(txt)

def goto(rm):
    # Goes to a room
    global room
    if rm in room:
        var['location'] = rm
        look()
    else:
        sys.stderr.write('! Error: Room "' + rm + '" does not exist.')

def look():
    # Will print description of current room and list available items
    global var, default
    croom = room[var['location']]
    if 'desc' in croom:
        p(croom['desc'])
    else:
        exec("p(" + default['room']['desc'] + ")")
    
    if not 'items' in croom:
        return 1
    
    global item
    for x in croom['items']:
        if x['id'] in item:
            citem = x['id'] # Descriptions use this variable
            if 'desc' in x:
                exec('p(' + x['desc'] + ')')
            else:
                exec('p(' + default['room']['item'] + ')')

def has(obj):
    global var
    return obj in var['inventory']

def isin(loc):
    global var
    return loc == var['location']

def getprefix(citem):
    # Gets article or prefix of item (a, an, some, etc.)
    if 'pref' in item[citem]:
        pref = item[citem]['pref']
    else:
        pref = 'a'
        if 'aeiou'.find(item[citem]['name'][0]) != -1:
            pref += 'n'
    return pref

def get_extra():
    pass # The text adventure currently run by Zita can utilize this function.

def exec_proper_command(cmd, i, i_or_o, citem):
    # This function will attempt to execute the correct command from a linking
    # command. If "take" links to "get", then this function find out just what
    # "get" version is the best. It follows a hierarchy:
    #   1. command in current item (citem) under either 'in' or 'out' parts of
    #      your item (depending on the i_or_o variable, which depends on the
    #      item's whereabouts. If it's in your inventory, then it's 'in'. Else
    #      it's 'out'.
    #   2. command in current item (citem) under 'cmd' part of your item.
    #   3. command in default item under i_or_o part of default item.
    #   4. command in default item under 'cmd' part of default item.
    
    # This way, all possible ways should be covered.
    global default
    ok = True
    di = default['item']
    if i_or_o in i and cmd in i[i_or_o]:
        exec(i[i_or_o][cmd])
    elif 'cmd' in i and cmd in i['cmd']:
        exec(i['cmd'][cmd])
    elif i_or_o in di and cmd in di[i_or_o]:
        exec(di[i_or_o][cmd])
    elif 'cmd' in di and cmd in di['cmd']:
        exec(di['cmd'][cmd])
    else:
        ok = False
    
    return ok

def inv(itm):
    # Add itm to inventory
    global var, room
    var['inventory'].append(itm)
    
    i = 0
    for x in room[var['location']]['items']:
        if x['id'] == itm:
            del room[var['location']]['items'][i]
        i += 1

def extra_add(txt):
    # Add code to extra variable and execute it
    global extra
    extra += '\n' + txt
    exec(txt)

def save(file):
    # Dump important globals
    global room, item, var, command, default, extra
    pickle.dump([room, item, var, command, default, extra], open(file, 'w'))

def load(file):
    # Do some pickle loading
    data = pickle.load(open(file, 'r'))
    global room, item, var, command, default, extra
    room = data[0]
    item = data[1]
    var = data[2]
    command = data[3]
    default = data[4]
    extra = data[5]
    
    toptext = 'global '
    for x in CURRENT:
        toptext += x + ', '
    
    toptext = toptext[:-2] + '\n' + open('defs.py').read() + '\n' + extra
    exec(toptext)
    if 'loaded' in default: exec(default['loaded'])
    look()
