#!/usr/bin/python
# -*- coding: utf-8 -*-

# Zita: A simple text adventure engine heavily relying on Python
# Copyright (C) 2009  Niels Serup
#  
#	This file is part of Zita.
#
#	Zita is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	Zita is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with Zita.  If not, see <http://www.gnu.org/licenses/>.

# You can contact me at ns@metanohi.org
# v0.1

import sys
if len(sys.argv) == 2:
    ok = True
    if sys.argv[1] == '-h' or sys.argv[1] == '--help':
        print """
            Zita  is a text  adventure  engine.
            Usage: zita [game directory = zeqy]
"""
    elif sys.argv[1] == '-v' or sys.argv[1] == '--version':
        print "v0.1"
    else:
        ok = False
    
    if ok:
        sys.exit()

import os, random, time, warnings
warnings.filterwarnings('ignore', ".*")

try: import cPickle as pickle # FASTER!
except: import pickle

defaultgamedir = 'zeqy'

if len(sys.argv) > 1:
    gamedir = sys.argv[1]
    while gamedir[-1:] == '/':
        gamedir = gamedir[:-1]
        if not gamedir:
            gamedir = defaultgamedir
else:
    gamedir = defaultgamedir

if not os.path.isdir(gamedir):
    sys.stderr.write('Game directory does not exist.\n')
    sys.exit()

# A few globals
mainfilename = 'main'
filesuffix = 'zt'
configfilename = 'prerun'
gamefiles = []
gamedirs = []


random.seed() # Pretty random
ok = False # This program is pessimistic

# Trying to load a config file in the game directory
if os.path.isfile(gamedir + '/' + configfilename):
    try:
        execfile(gamedir + '/' + configfilename)
        ok = True
    except:
        pass

if not ok:
    # If no config file in game dir, maybe one in Zita dir?
    if os.path.isfile(configfilename):
        try:
            execfile(configfilename)
            ok = True
        except:
            ok = False
    if not ok:
        f = open(configfilename, 'w')
        f.write('# This is the config file for Zita. The variables and values in this file will be loaded when Zita starts. Note that this file is not loaded if a config file in the game directory of the current game is present.\n\n# Assigning custom values to other variables than the following is not recommended, as it might crash Zita.\n# * mainfilename -- the name of the main file used in a game\n# * filesuffix -- the file suffix generally used by files in a game\n\n#mainfilename = \'index\'\n# By removing the comment sign (#) in the above line, Zita will override its program defaults for the variable mainfilename.')


# FUNCTIONS #
try:
    execfile('defs.py')
except:
    sys.stderr.write('Basic functions do not exist.\n')
    sys.exit()



# MAIN #

mainfile = getfilenames(mainfilename, 'file')
if not mainfile:
    sys.stderr.write('Main file does not exist.')
    sys.exit()

# Clear screen
try: os.system('clear') # Works in a bash shell
except: os.system('cls') # Windows

# Define global storage variables
room, item, var, command, default, extra = {}, {}, {}, {}, {}, ''


# Soon Zita will start to read EVERYTHING associated with the current game.
# Both analyzing, perspectivating, crylotozatoting and processing will be done.
# The end result will be magnificent.
execfile(mainfile)

chosen_item = None
COMPLETELY_DONE = False
while not COMPLETELY_DONE:
    # Will get and process input
    CURRENT = dir() # Used by gameloading mechanism
    txt = raw_input('> ').lower()
    if not txt:
        continue
    cmd = txt.split(' ')
    
    ok = False # Perhaps
    
    # The below code first checks if the command consist of more than one word.
    # If that's the case, get() will start to search for item commands, as such
    # commands always need at least two words (like "cook duck").
    
    # If no item commands match, the ok variable remains set to False.
    
    if len(cmd) > 1: # Like: "eat monster" or "invade Martian grounds"
        if cmd[1] not in default['item-substitutes']:
            ml = len(cmd[0]) + 1 # The start of the rest of the command        
            what = txt[ml:] # The rest of the command
            # Example: In "catch moon", ml would be 6, and what would be "moon".
            
            poss_items = [] # None yet
            di = default['item'] # For default actions
            
            # Items in inventory
            for x in var['inventory']:
                i = item[x]
                name = i['name']
                if what == name[-len(what):] and \
                 (('cmd' in i and cmd[0] in i['cmd']) or ('in' in i and cmd[0] in i['in']) or \
                  ('cmd' in di and cmd[0] in di['cmd']) or ('in' in di and cmd[0] in di['in'])):
                    poss_items.append([x, 1])
            
            # Items in current location
            if 'items' in room[var['location']]:
                for x in room[var['location']]['items']:
                    i = item[x['id']]
                    name = i['name']
                    if what == name[-len(what):] and \
                     (('cmd' in i and cmd[0] in i['cmd']) or ('out' in i and cmd[0] in i['out']) or \
                      ('cmd' in di and cmd[0] in di['cmd']) or ('out' in di and cmd[0] in di['out'])):
                        poss_items.append([x['id'], 0])
            
            if len(poss_items) == 0:
                chosen_item = None
            elif len(poss_items) == 1:
                chosen_item = poss_items[0]
            else:
                # Ask user to specify item
                exec('which = ' + var['which'])
                exec('invalidwhich = ' + var['invalidwhich'])
                chosen_item = None # Until user makes a choice
                while not chosen_item:
                    p(which)
                    i = 0
                    for x in poss_items:
                        p(var['listoptions'][i] + '. ' + item[x[0]]['name'])
                        i += 1
                    nt = raw_input('> ')
                    for c in range(i):
                        if nt == var['listoptions'][c]:
                            chosen_item = poss_items[c]
                            break
                    if not chosen_item:
                        p(invalidwhich)
        
        if chosen_item:
            i = item[chosen_item[0]]
            citem = chosen_item[0]
            if chosen_item[1]:
                # If item in inventory
                i_or_o = 'in'
            else:
                i_or_o = 'out'
            
            ok = exec_proper_command(cmd[0], i, i_or_o, citem) # Exciting
    
    if not ok:
        # Then check if command exists as a command-type command instead.
        if cmd[0] in command:
            exec(command[cmd[0]])
            ok = True
    
    if not ok:
        # Default error message
        exec(default['command'])
    
    get_extra() # For manipulation purposes
    if 'steps' in var: var['steps'] += 1
